/*  MSS -- Memory Supervision System version 1.0B
 *  Copyright (C) 1998  Juan Jesus Alcolea Picazo and Peter Palotas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  You can contact the authors of this library at the following e-mail
 *  addreses. For more information, look in the documentation that came
 *  with this library.
 *
 *  Juan Jesus Alcolea Picazo, a920101@zipi.fi.upm.es
 *  Peter Palotas, blizzar@hem1.passagen.se
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include "mss.h"

void Happy_Waster(int quantity)
{
        void *dustbin=(new char[quantity]);  //Yeah! waste it all!! ]:>
        (void)dustbin;
}

/* This is a *simple* test program written to show some MSS features. In
   order to test all of them, please enable WATCH_LIMITS and MEM_INIT in
   mss.h and recompile. Refer to MSS.DOC for more detailed info. */
class MssGlobalCtorClass
{
private:
	char *pointer;
public:
	MssGlobalCtorClass(void)
	{
	 	pointer = new char[14];
	}

	~MssGlobalCtorClass(void)
	{
 	 	delete[] pointer;
	}
};

MssGlobalCtorClass this_works_too;

void exit_test(void)
{
	int *i = new int;
	delete i;
	MSS_LOG_MSG("If this message is the last line in the log-file, "
 	            "you should try toggeling the compiler specific option "
		    "MSS_DTOR_AFTER_ATEXIT in `mss.h'.");
}

int main(void)
{
 	int z = 0;
	char *useless;                 // This is not going to be initialized
	char *tonowhere = NULL;          // points to NULL
	int *zero_expected;
	zero_expected = new int;    // I supposse this to be initialized to zero...
	printf("%i\n", *zero_expected); // What! MSS initialized it to 0x4d4d4d4d4d...

	atexit(&exit_test);

	Happy_Waster(100);            //I call this nice function

	/* Do some tests for ex. deleting malloced memory and so on. */
	char *malloced = (char *)malloc(10);
	char *realloced = (char *)realloc(malloced, 20);
	char *calloced = (char *)calloc(300, sizeof(char));
	char *newed = new char[100];

	free(newed); // MSS will warn about this.
	delete malloced; // deleting unallocated memory (probably)
	delete realloced; // deleting memory allocated by realloc.
	delete calloced;  // this isn't good either.

	newed = new char[100];
	newed = (char *)realloc(newed, 50); // A warning should be printed about this too

	char *str1 = new char[50];      // 50 chars from [0] to [49]
	char *str2 = new char[100];
	str2--;                        // we are stepping out of our legal zone...
	MSS_CHECK_POINTER_VALIDITY(str2);           // MSS warns about this.

	str1[50]=0;               // Ooops, we are writing *out* range!
	str1[-1]=0;               // Same, but in the opposite direction...

	delete[] str1;       //MSS checks and detects both 'out of range' writings
	delete zero_expected; //Now we have freed all memory, didn't we?
	printf("%i\n", *zero_expected); // What! MSS 'finished' it to 0x88888888...
	MSS_LOG_INFO;             //Let's ask MSS... hey! There are still 1000 bytes allocated!
	MSS_LOG_BLOCK_LIST;       //Give me some info about all allocated memory...
	                      //oh, is that Happy_Waster function who allocated this mem...

	delete [] str1;       //I deleted this before. MSS will catch and warn this also.
	delete [] useless;    //This was never allocated. MSS warns this too.

	MSS_ENTER_SCOPE;
	MSS_DISABLE_LOG_OUTPUT;   //I don't want next 100 allocations to be logged, but i still
	for(int k = 0; k < 100; k++) //want MSS to proccess them.
		Happy_Waster(rand() % 100); // <-- This is nice programming practice, isn't it? ;)
	MSS_ENABLE_LOG_OUTPUT;    //OK, log everything again to the output file.
	MSS_LEAVE_SCOPE; 	// We didn't really expect anyone to have
				// deallocated all memory here, did we!? ;)

	MSS_LOG_INFO;             //Tell me what's the situation
	MSS_LOG_BLOCK_LIST;       //Show me all allocated blocks
        MSS_LOG_INTERNAL_INFO;    //Show some info about MSS Internals

	str1 = new char[z];    //Zero length allocations are also catched and warned by MSS
	MSS_LOG_MSG("Hello from source! How goes everything?");

	delete tonowhere;    //Wait! This points to NULL...!
	free(tonowhere);
	return 0;
}


