/*  MSS -- Memory Supervision System version 1.0B
 *  Copyright (C) 1998  Juan Jesus Alcolea Picazo and Peter Palotas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  You can contact the authors of this library at the following e-mail
 *  addreses. For more information, look in the documentation that came
 *  with this library.
 *
 *  Juan Jesus Alcolea Picazo, a920101@zipi.fi.upm.es
 *  Peter Palotas, blizzar@hem1.passagen.se
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "mss.h"

void Happy_Waster(int quantity)
{
        void *dustbin=(void *)malloc(quantity);  /* Yeah! waste it all!! ]:> */
	MSS_SET_BLOCK_LABEL(dustbin, "dustbin");
        (void)dustbin;
}

int main(void)
{
 	int z = 0, k;
	char *useless;     /* This is not going to be initialized */
	char *tonowhere = NULL;          /* points to NULL */
	int *zero_expected;
	char *str1, *str2;
	int i;
 	zero_expected = (int *)malloc(sizeof(int));    /* I supposse this to be initialized to zero...  */
	MSS_SET_BLOCK_LABEL(zero_expected, "int zero_expected");
	printf("%i\n", *zero_expected); /* What! MSS initialized it to 0x4d4d4d4d4d... */

	MSS_STARTUP;

	Happy_Waster(100);            /* I call this nice function */

	str1 = (char *)malloc(50 * sizeof(char));      /* 50 chars from [0] to [49] */
	MSS_SET_BLOCK_LABEL(str1, "str1");
	str2 = (char *)malloc(100 * sizeof(char));
	MSS_SET_BLOCK_LABEL(str2, "str2");
	str2--;                        /* we are stepping out of our legal zone... */
        MSS_CHECK_POINTER_VALIDITY(str2);           /* MSS warns about this. */

	str1[50]=0;               /* Ooops, we are writing *out* range! */
	str1[-1]=0;               /* Same, but in the opposite direction... */

	free(str1);       /* MSS checks and detects both 'out of range' writings */
	free(zero_expected); /* Now we have freed all memory, didn't we? */
	printf("%i\n", *zero_expected); /* What! MSS 'finished' it to 0x88888888... */
	MSS_LOG_INFO;             /*Let's ask MSS... hey! There are still 1000 bytes allocated! */
	MSS_LOG_BLOCK_LIST;       /*Give me some info about all allocated memory... */
	                      /*oh, is that Happy_Waster function who allocated this mem... */

	free(str1);       /*I deleted this before. MSS will catch and warn this also. */
	free(useless);    /*This was never allocated. MSS warns this too. */

	MSS_DISABLE_LOG_OUTPUT;   /* I don't want next 100 allocations to be logged, but i still */
	for(k = 0; k < 100; k++) /* want MSS to proccess them. */
		Happy_Waster(rand() % 100); /* <-- This is nice programming practice, isn't it? ;)  */
	MSS_ENABLE_LOG_OUTPUT;    /* OK, log everything again to the output file. */

	MSS_LOG_INFO;             /* Tell me what's the situation */
	MSS_LOG_BLOCK_LIST;       /* Show me all allocated blocks */

	str1 = (char *)malloc(z * sizeof(char));    /* Zero length allocations are also catched and warned by MSS */
	MSS_LOG_MSG("Hello from source! How goes everything?");

	free(tonowhere);    /* Wait! This points to NULL...! */

	str1 = (char *)xmalloc(28);
	for (i = 0; i < 28; i++)
		str1[i] = 'A' + i;
	MSS_DUMP_BLOCK(str1, "str1.dmp");
	xfree(str1);


	return 0;
}

